@@ -5,6 +5,7 @@ This collects the general public API into a single spot for inclusion, as
 !*/
 
 pub use crate::{
+	access::BitAccess,
 	bits::{
 		Bits,
 		BitsMut,
@@ -13,9 +14,17 @@ pub use crate::{
 		Cursor,
 		BigEndian,
 		LittleEndian,
+		Local,
+	},
+	fields::BitField,
+	slice::{
+		BitMut,
+		BitSlice,
+	},
+	store::{
+		BitStore,
+		Word,
 	},
-	slice::BitSlice,
-	store::BitStore,
 };
 
 #[cfg(feature = "alloc")]
